#!/bin/bash
#
#   This script is based on a similar script in Dropspace that generates Apache configuration files.
#
#   This script will, for each directory in the Web API Bridge domains directory,
#   create an Apache2 configuration file called 'WEB_API_BRIDGE.<domain name>.conf',
#   which is a copy of the 'WEB_API_BRIDGE.TEMPLATE.conf' file, that has each occurrance
#   of the pattern %domain_name% replaced with <domain name>.
#
#   It needs to know:
#
#   1)  Where the domains directory is, i.e. /tmp/web_api_bridge, and
#   2)  The location of the Apache configuration directory (/etc/apache2 by default).

function usage()
{
    echo "Usage: generate_webapibridge_apache_configurations.sh <Document Root> [Apache config dir]"
    exit -1
}

#
#   Utility functions
#

function scriptname()
{
    echo $(readlink -f $0)
}

function main()
{
    local document_root=$1
    local apache_config_dir=$2
    local domains_dir="/tmp/web_api_bridge"

    echo ""
    echo `date`

    if [ ! -d "$document_root" ]
    then

        echo "Invalid Document Root dir: $document_root"
        usage

    else

        if [ -z "$apache_config_dir" ]
        then

            echo "Using default Apache config dir"
            apache_config_dir="/etc/apache2"

        elif [ ! -d "$apache_config_dir" ]
        then

            echo "Invalid Apache config dir"
            usage

        fi

   echo process_subdirectories "$document_root" "$apache_config_dir"
        process_subdirectories "$document_root" "$apache_config_dir"

    fi
}

function process_subdirectories()
{
    local script_name=`scriptname`
    local script_dir=`dirname "$script_name"`
    local libexec_dir=`dirname "$script_dir"`
    local dropspace_dir=`dirname "$libexec_dir"`
    local latest_dir=`dirname "$dropspace_dir"`
    local DropSpace_dir=`dirname "$latest_dir"`

    local apache_template_dir="$dropspace_dir/configuration/apache2"
    local apache_template_file="$apache_template_dir/WEB_API_BRIDGE.TEMPLATE.conf"

    local document_root=$1
    if [ -z "$document_root" ]
    then

        document_root=`dirname "$DropSpace_dir"`

    elif [ ! -d "$document_root" ]
    then

        echo "Invalid Document root"
        usage

    fi

    local apache_config_dir=$2
    local target="$apache_config_dir/sites-enabled"

    if [ ! -f "$apache_template_file" ]
    then

        echo "Internal error: could not find Apache configuration template file at: $apache_template_file"
        exit

    else

        for domain_dir in $( ls "$domains_dir" )
        do

            if [ ! -f "$target/WEB_API_BRIDGE.$domain_dir.conf" -a ! -f "$target/WEB_API_BRIDGE.$domain_dir-le-ssl.conf" -a ! -f "$target/WEB_API_BRIDGE.$domain_dir.ssl.conf" ]
            then

                local domain_name="$domain_dir"
                local subdomain="${domain_dir/api-/}"
                      subdomain="${domain_dir/api./}"

                local tld=`echo "$domain_dir" | cut -f1 -d'.'`

                if is_tld $tld
                then
                    domain_name=`reverse_domain $domain_dir`

                fi

                echo "Processing: $domain_dir for $domain_name"
                echo "Creating:   $target/WEB_API_BRIDGE.$domain_dir.conf"
                sed "s|%document_root%|$document_root|g" "$apache_template_dir/WEB_API_BRIDGE.TEMPLATE.conf" | sed "s|%domain_name%|$domain_name|g" | sed "s|%domain_dir%|$domain_dir|g" | sed "s|%subdomain%|$subdomain|g" > "$target/WEB_API_BRIDGE.$domain_dir.conf"

#                if string_ends_with "$domain_name" ".local"
#                then
#
#                    if [ -f "/etc/apache2/ssl/wildcard.dropspace.local.pem" ]
#                    then
#
#                        if [ ! -f "$target/WEB_API_BRIDGE.$domain_dir.ssl.conf" ]
#                        then
#
#                            echo "Creating:   $target/WEB_API_BRIDGE.$domain_dir.ssl.conf"
#                            sed "s|%document_root%|$document_root|g" "$apache_template_dir/WEB_API_BRIDGE.TEMPLATE.SSL.conf" | sed "s|%domain_name%|$domain_name|g" | sed "s|%domain_dir%|$domain_dir|g" > "$target/DROPSPACE.$domain_dir.ssl.conf"
#
#                        fi
#                    fi
#                fi
            fi
        done
    fi
}

function string_ends_with()
{
    local haystack=$1
    local suffix=$2

    if [ -z "${haystack##*$suffix}" ]
    then
        return  0 # success
    else
        return -1 # failure
    fi
}

function reverse_domain()
{
    local dir=$1
    local is_local="FALSE"

    #
    #   If domain has .local suffix remove and remember.
    #
    if string_ends_with $dir .local
    then
        is_local="TRUE"
        dir=${dir/.local/}                      # com.domain.local --> com.domain
    fi

    local parts=${dir//./ }                     # com.domain       --> com domain

    local reversed=`reverse_arguments $parts`   # com domain       --> domain com
    local domain=${reversed// /.}               # domain com       --> domain.com

    #
    #   If had .local suffix reattach.
    #
    if [ "TRUE" == $is_local ]
    then
        domain+=".local"
    fi

    echo $domain
}

function reverse_arguments()
{
        local mine=$1

        if [ $# -gt 1 ]
        then
            shift
            reverse_arguments $@
            printf " $mine"
        else
            printf "$mine"
        fi
}

function is_tld()
{
    case $1 in
        com|net|org|info)
            return  0 # true
            ;;

        au|nz|uk|us)
            return  0 # true
            ;;

        online|xyz)
            return  0 # true
            ;;

        *)
            return -1 # false
    esac
}

if [ $# -lt 1 ]
then

    usage

else

    main $@

fi
